package io.adjoe.cocos2dx;

import android.content.Intent;
import android.support.annotation.Nullable;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import io.adjoe.sdk.Adjoe;
import io.adjoe.sdk.AdjoeException;
import io.adjoe.sdk.AdjoeGender;
import io.adjoe.sdk.AdjoeInitialisationListener;
import io.adjoe.sdk.AdjoeNotInitializedException;
import io.adjoe.sdk.AdjoeParams;
import io.adjoe.sdk.AdjoePayoutError;
import io.adjoe.sdk.AdjoePayoutListener;
import io.adjoe.sdk.AdjoeRewardListener;
import io.adjoe.sdk.AdjoeRewardResponse;
import io.adjoe.sdk.AdjoeRewardResponseError;
import io.adjoe.sdk.AdjoeUserProfile;

import static org.cocos2dx.lib.Cocos2dxActivity.getContext;

public class AdjoeSDKModule {

    private AdjoeSDKModule() {}

    public static native void doLuaCallback(final int functionReference, @Nullable final String message);

    public static native void doLuaRewardCallback(final int functionReference, final AdjoeRewardResponse response);

    public static native void doLuaPayoutSuccessCallback(final int functionReference, final int coins);

    public static native void doLuaPayoutErrorCallback(final int functionReference, final int reason, final String message);

    public static native void freeCallbackReferences(final int... callbackRefs);

    public static void doPayout(final AdjoeOptions options, final int luaSuccessCallback, final int luaErrorCallback) {
        try {
            Adjoe.doPayout(getContext(), options != null ? options.toParams() : null, new AdjoePayoutListener() {

                @Override
                public void onPayoutExecuted(int i) {
                    try {
                        doLuaPayoutSuccessCallback(luaSuccessCallback, i);
                    } finally {
                        freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
                    }
                }

                @Override
                public void onPayoutError(AdjoePayoutError e) {
                    try {
                        final Exception ex = e.getException();
                        doLuaPayoutErrorCallback(luaErrorCallback, e.getReason(), ex != null ? ex.toString() : "");
                    } finally {
                        freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
                    }
                }
            });
        } catch (AdjoeNotInitializedException e) {
            try {
                doLuaPayoutErrorCallback(luaErrorCallback, 0, e.toString());
            } finally {
                freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
            }
        }
    }

    public static String sendEvent(final int event, final AdjoeOptions options) {
        try {
            Adjoe.sendUserEvent(getContext(), event, null, options != null ? options.toParams() : null);
            return null;
        } catch (AdjoeNotInitializedException e) {
            return e.toString();
        }
    }

    public static void initAdjoe(final String sdkHash, AdjoeOptions options, final int luaSuccessCallback, final int luaErrorCallback) {
        Adjoe.init(getContext(), sdkHash, options != null ? options.toOptions() : null, new AdjoeInitialisationListener() {

            @Override
            public void onInitialisationFinished() {
                try {
                    doLuaCallback(luaSuccessCallback, null);
                } finally {
                    freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
                }
            }

            @Override
            public void onInitialisationError(Exception e) {
                try {
                    doLuaCallback(luaErrorCallback, e.toString());
                } finally {
                    freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
                }
            }
        });
    }

    public static void requestRewards(final AdjoeOptions options, final int luaSuccessCallback, final int luaErrorCallback) {
        try {
            Adjoe.requestRewards(getContext(), options != null ? options.toParams() : null, new AdjoeRewardListener() {

                @Override
                public void onUserReceivesReward(AdjoeRewardResponse adjoeRewardResponse) {
                    try {
                        doLuaRewardCallback(luaSuccessCallback, adjoeRewardResponse);
                    } finally {
                        freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
                    }
                }

                @Override
                public void onUserReceivesRewardError(AdjoeRewardResponseError e) {
                    try {
                        doLuaCallback(luaErrorCallback, e.getException().toString());
                    } finally {
                        freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
                    }
                }
            });
        } catch (AdjoeNotInitializedException e) {
            try {
                doLuaCallback(luaErrorCallback, e.toString());
            } finally {
                freeCallbackReferences(luaSuccessCallback, luaErrorCallback);
            }
        }
    }

    @Nullable
    public static String setProfile(final String source, final String gender, final String birthday, final AdjoeOptions options) {
        try {
            final String adjoeSource = source != null ? source : "";

            final AdjoeGender adjoeGender;
            switch (gender) {
                case "male":
                    adjoeGender = AdjoeGender.MALE;
                    break;
                case "female":
                    adjoeGender = AdjoeGender.FEMALE;
                    break;
                default:
                    adjoeGender = AdjoeGender.UNKNOWN;
                    break;
            }

            final Date adjoeBirthday = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).parse(birthday);
            final AdjoeParams adjoeParams = options != null ? options.toParams() : null;
            Adjoe.setProfile(getContext(), adjoeSource, new AdjoeUserProfile(adjoeGender, adjoeBirthday), adjoeParams);
            return null;
        } catch (Exception e) {
            return e.toString();
        }
    }

    @Nullable
    public static String showOfferwall(final AdjoeOptions options) {
        try {
            Intent intent = Adjoe.getOfferwallIntent(getContext(), options != null ? options.toParams() : null);
            getContext().startActivity(intent);
            return null;
        } catch (AdjoeException e) {
            return e.toString();
        }
    }

    public static boolean isInitialized() {
        return Adjoe.isInitialized();
    }

    public static boolean canShowOfferwall() {
        return Adjoe.canShowOfferwall(getContext());
    }

    public static boolean hasAcceptedTOS() {
        return Adjoe.hasAcceptedTOS(getContext());
    }

    public static boolean hasAcceptedUsagePermission() {
        return Adjoe.hasAcceptedUsagePermission(getContext());
    }

    public static int getVersion() {
        return Adjoe.getVersion();
    }

    public static String getVersionName() {
        return Adjoe.getVersionName();
    }
}
